<?php

/**
 * JCH Optimize - Performs several front-end optimizations for fast downloads
 *
 *  @package   jchoptimize/core
 *  @author    Samuel Marshall <samuel@jch-optimize.net>
 *  @copyright Copyright (c) 2022 Samuel Marshall / JCH Optimize
 *  @license   GNU/GPLv3, or later. See LICENSE file
 *
 *  If LICENSE file missing, see <http://www.gnu.org/licenses/>.
 */
namespace JchOptimize\Core\Model;

use FilesystemIterator;
use JchOptimize\Core\PageCache\CaptureCache;
use JchOptimize\Platform\Paths;
use _JchOptimizeVendor\Joomla\Filesystem\Folder;
use _JchOptimizeVendor\Laminas\Cache\Exception\ExceptionInterface;
use _JchOptimizeVendor\Laminas\Cache\Storage\FlushableInterface;
use _JchOptimizeVendor\Laminas\Cache\Storage\IterableInterface;
use _JchOptimizeVendor\Laminas\Cache\Storage\TaggableInterface;
use RecursiveDirectoryIterator;
use RecursiveIteratorIterator;
use function file_exists;
use function filesize;
use function floor;
use function in_array;
use function is_array;
use function iterator_count;
use function md5;
use function number_format;
use function pow;
use function sprintf;
use function strlen;
trait CacheModelTrait
{
    public function getCacheSize() : array
    {
        $size = 0;
        $numFiles = 0;
        if ($this->cache instanceof IterableInterface) {
            try {
                $iterator = $this->cache->getIterator();
                $numFiles += iterator_count($iterator);
                foreach ($iterator as $item) {
                    //Let's skip the 'test' cache set on instantiation in container
                    if ($item == md5('__ITEM__')) {
                        $numFiles -= 1;
                        continue;
                    }
                    $metaData = $this->cache->getMetadata($item);
                    if (!is_array($metaData)) {
                        continue;
                    }
                    if (isset($metaData['size'])) {
                        $size += $metaData['size'];
                    } else {
                        if (file_exists($metaData['filespec'] . '.dat')) {
                            $size += filesize($metaData['filespec'] . '.dat');
                        }
                    }
                }
            } catch (ExceptionInterface|\Exception $e) {
            }
        }
        //Iterate through the static files
        if (file_exists(Paths::cachePath(\false))) {
            $directory = new RecursiveDirectoryIterator(Paths::cachePath(\false), FilesystemIterator::SKIP_DOTS);
            $iterator = new RecursiveIteratorIterator($directory);
            $i = 0;
            foreach ($iterator as $file) {
                if (in_array($file->getFilename(), ['index.html', '.htaccess'])) {
                    $i++;
                    continue;
                }
                $size += $file->getSize();
            }
            $numFiles += iterator_count($iterator) - $i;
        }
        $decimals = 2;
        $sz = 'BKMGTP';
        $factor = (int) floor((strlen($size) - 1) / 3);
        $size = sprintf("%.{$decimals}f", $size / pow(1024, $factor)) . $sz[$factor];
        $numFiles = number_format($numFiles);
        return [$size, $numFiles];
    }
    /**
     * Cleans cache from the server
     *
     * @return bool
     */
    public function cleanCache() : bool
    {
        $success = \false;
        //Clean all cache generated by Storage
        if ($this->cache instanceof FlushableInterface) {
            $success |= $this->cache->flush();
        }
        //Also delete tags
        $taggableCache = $this->container->get(TaggableInterface::class);
        $taggableCache->flush();
        if (\JCH_PRO) {
            //Perhaps Capture Cache was used, so we'll need to delete capture cache
            /** @var CaptureCache $captureCache */
            $captureCache = $this->container->get(CaptureCache::class);
            $captureCache->deleteAllItems();
        }
        //Delete any static combined files
        $staticCachePath = Paths::cachePath(\false);
        try {
            if (file_exists($staticCachePath . '/css')) {
                $success |= Folder::delete($staticCachePath . '/css');
            }
            if (file_exists($staticCachePath . '/js')) {
                $success |= Folder::delete($staticCachePath . '/js');
            }
        } catch (\Exception $e) {
            $success |= \false;
        }
        return $success;
    }
}
