<?php

/**
 * JCH Optimize - Performs several front-end optimizations for fast downloads
 *
 * @package   jchoptimize/core
 * @author    Samuel Marshall <samuel@jch-optimize.net>
 * @copyright Copyright (c) 2022 Samuel Marshall / JCH Optimize
 * @license   GNU/GPLv3, or later. See LICENSE file
 *
 *  If LICENSE file missing, see <http://www.gnu.org/licenses/>.
 */
namespace JchOptimize\Core\Plugin;

use JchOptimize\Core\PageCache\PageCache;
use JchOptimize\Platform\Paths;
use JchOptimize\Platform\Profiler;
use _JchOptimizeVendor\Joomla\DI\ContainerAwareInterface;
use _JchOptimizeVendor\Joomla\DI\ContainerAwareTrait;
use _JchOptimizeVendor\Joomla\Filesystem\File;
use _JchOptimizeVendor\Joomla\Registry\Registry;
use _JchOptimizeVendor\Laminas\Cache\Exception\ExceptionInterface;
use _JchOptimizeVendor\Laminas\Cache\Storage\Plugin\AbstractPlugin;
use _JchOptimizeVendor\Laminas\Cache\Storage\PostEvent;
use _JchOptimizeVendor\Laminas\Cache\Storage\StorageInterface;
use _JchOptimizeVendor\Laminas\Cache\Storage\TaggableInterface;
use _JchOptimizeVendor\Laminas\EventManager\EventManagerInterface;
use Psr\Log\LoggerAwareInterface;
use Psr\Log\LoggerAwareTrait;
use function count;
use function file_exists;
use function random_int;
use function time;
class ClearExpiredByFactor extends AbstractPlugin implements ContainerAwareInterface, LoggerAwareInterface
{
    use ContainerAwareTrait;
    use LoggerAwareTrait;
    public function attach(EventManagerInterface $events, $priority = 1)
    {
        $callback = [$this, 'clearExpiredByFactor'];
        $this->listeners[] = $events->attach('setItem.post', $callback, $priority);
        $this->listeners[] = $events->attach('setItems.post', $callback, $priority);
        $this->listeners[] = $events->attach('addItem.post', $callback, $priority);
        $this->listeners[] = $events->attach('addItems.post', $callback, $priority);
    }
    /**
     * @throws \Exception
     */
    public function clearExpiredByFactor(PostEvent $event)
    {
        $factor = $this->getOptions()->getClearingFactor();
        if ($factor && random_int(1, $factor) === 1) {
            $this->clearExpired();
        }
    }
    private function clearExpired()
    {
        JCH_DEBUG ? Profiler::start('ClearExpired') : null;
        /** @var Registry $params */
        $params = $this->container->get('params');
        /** @var TaggableInterface $taggableCache */
        $taggableCache = $this->container->get(TaggableInterface::class);
        /** @var StorageInterface $cache */
        $cache = $this->container->get(StorageInterface::class);
        /** @var PageCache $pageCache */
        $pageCache = $this->container->get(PageCache::class);
        $ttl = $cache->getOptions()->getTtl();
        $ttlPageCache = $pageCache->getStorage()->getOptions()->getTtl();
        $time = time();
        foreach ($taggableCache->getIterator() as $item) {
            $tags = $taggableCache->getTags($item);
            $metaData = $taggableCache->getMetadata($item);
            $mtime = $metaData['mtime'];
            //If there are only one tag this cache was only used on one page so is safe to delete if expired
            //Or if the cache is now more than twice it's lifetime it's time to say bye.
            if (count($tags) === 1 && $time >= $mtime + $ttl || $time > $mtime + 2.5 * $ttl) {
                try {
                    //Remove cache
                    $cache->removeItem($item);
                } catch (ExceptionInterface $e) {
                    //Don't bother to remove tags if this didn't work, we'll try again next time
                    continue;
                }
                //We need to also delete the static css/js file if that option is set
                if ($params->get('htaccess', '2') == '2') {
                    $cssUrl = Paths::cachePath(\false) . '/css/' . $item . '.css';
                    $jsUrl = Paths::cachePath(\false) . '/js/' . $item . '.js';
                    try {
                        if (file_exists($cssUrl)) {
                            File::delete($cssUrl);
                        }
                        if (file_exists($jsUrl)) {
                            File::delete($jsUrl);
                        }
                    } catch (\Exception $e) {
                        //Don't bother to delete tag if this didn't work, we'll try again next time.
                        continue;
                    }
                }
                //Remove tag if cache successfully deleted
                $taggableCache->removeItem($item);
            }
            //If page cache just delete if expired
            if ($tags[0] == 'pagecache' && $time >= $mtime + $ttlPageCache) {
                //Remove cache and tags
                $pageCache->deleteItemById($item);
            }
        }
        JCH_DEBUG ? Profiler::stop('ClearExpired', \true) : null;
    }
}
