<?php

/**
 * JCH Optimize - Performs several front-end optimizations for fast downloads
 *
 * @package   jchoptimize/wordpress-platform
 * @author    Samuel Marshall <samuel@jch-optimize.net>
 * @copyright Copyright (c) 2022 Samuel Marshall / JCH Optimize
 * @license   GNU/GPLv3, or later. See LICENSE file
 *
 * If LICENSE file missing, see <http://www.gnu.org/licenses/>.
 */
namespace JchOptimize\Model;

use JchOptimize\Core\PageCache\CaptureCache;
use JchOptimize\Core\PageCache\PageCache as CorePageCache;
use _JchOptimizeVendor\Joomla\DI\Container;
use _JchOptimizeVendor\Joomla\DI\ContainerAwareInterface;
use _JchOptimizeVendor\Joomla\DI\ContainerAwareTrait;
use _JchOptimizeVendor\Joomla\Input\Input;
use _JchOptimizeVendor\Joomla\Model\StatefulModelInterface;
use _JchOptimizeVendor\Joomla\Model\StatefulModelTrait;
use _JchOptimizeVendor\Joomla\Registry\Registry;
use _JchOptimizeVendor\Laminas\Cache\Storage\Adapter\Filesystem;
use _JchOptimizeVendor\Laminas\Cache\Storage\StorageInterface;
class PageCache implements StatefulModelInterface, ContainerAwareInterface
{
    use StatefulModelTrait;
    use ContainerAwareTrait;
    /**
     * @var StorageInterface
     */
    private $cache;
    /**
     * @var Filesystem
     */
    private $fsCache;
    /**
     * name of currently used cache adapter
     *
     * @var string
     */
    private $adapter;
    /**
     * @var CorePageCache
     */
    private $pageCache;
    public function __construct(Input $input, CorePageCache $pageCache, Container $container)
    {
        $this->pageCache = $pageCache;
        $this->setContainer($container);
        if (\JCH_PRO) {
            /** @see CaptureCache::updateHtaccess() */
            $this->container->get(CaptureCache::class)->updateHtaccess();
        }
        try {
            $registry = $this->populateRegistryFromRequest($input, ['filter', 'list']);
        } catch (\Exception $e) {
            $registry = new Registry();
        }
        $this->setState($registry);
    }
    private function populateRegistryFromRequest($input, array $keys)
    {
        $data = new Registry();
        foreach ($keys as $key) {
            //Check for value from input first
            $requestKey = $input->get($key);
            if (\is_null($requestKey)) {
                //Not found, let's check the transients
                $requestKey = \get_transient('jch_optimize_state_' . $key);
            }
            //If we've got one by now let's set it in registry
            if (!empty($requestKey) && \is_array($requestKey)) {
                foreach ($requestKey as $requestName => $requestValue) {
                    if (!empty($requestValue) || $requestValue == '0') {
                        $data->set($key . '_' . $requestName, $requestValue);
                    }
                }
                //Let's save this one in the transient
                set_transient('jch_optimize_state_' . $key, $requestKey, '300');
            }
        }
        return $data;
    }
    public function getItems() : array
    {
        $filters = ['time-1', 'time-2', 'search', 'device', 'adapter', 'http-request'];
        foreach ($filters as $filter) {
            $filterState = $this->getState()->get("filter_{$filter}");
            if (!empty($filterState)) {
                $this->pageCache->setFilter("filter_{$filter}", $filterState);
            }
        }
        //ordering
        $fullOrderingList = $this->getState()->get('list_fullordering');
        if (!empty($fullOrderingList)) {
            $this->pageCache->setList('list_fullordering', $fullOrderingList);
        }
        return $this->pageCache->getItems();
    }
    public function delete($ids) : bool
    {
        return $this->pageCache->deleteItemsByIds($ids);
    }
    public function deleteAll() : bool
    {
        return $this->pageCache->deleteAllItems();
    }
    public function getAdapterName() : string
    {
        return $this->pageCache->getAdapterName();
    }
    public function isCaptureCacheEnabled() : bool
    {
        return $this->pageCache->isCaptureCacheEnabled();
    }
}
