<?php

/**
 * JCH Optimize - Performs several front-end optimizations for fast downloads
 *
 * @package   jchoptimize/wordpress-platform
 * @author    Samuel Marshall <samuel@jch-optimize.net>
 * @copyright Copyright (c) 2020 Samuel Marshall / JCH Optimize
 * @license   GNU/GPLv3, or later. See LICENSE file
 *
 * If LICENSE file missing, see <http://www.gnu.org/licenses/>.
 */
namespace JchOptimize\Plugin;

use JchOptimize\Container;
use JchOptimize\Core\Admin\Tasks;
use JchOptimize\Core\Exception;
use JchOptimize\Core\Helper;
use JchOptimize\Core\Optimize;
use JchOptimize\Core\PageCache\PageCache;
use JchOptimize\Core\SystemUri;
use JchOptimize\Model\Cache;
use JchOptimize\Platform\Utility;
use _JchOptimizeVendor\Joomla\DI\ContainerAwareInterface;
use _JchOptimizeVendor\Joomla\DI\ContainerAwareTrait;
use _JchOptimizeVendor\Joomla\Input\Input;
use _JchOptimizeVendor\Joomla\Registry\Registry;
use Psr\Log\LoggerAwareInterface;
use Psr\Log\LoggerAwareTrait;
use Psr\Log\LogLevel;
use function delete_option;
use function add_action;
use function add_filter;
use function apply_filters;
use function get_option;
use function is_admin;
use function load_plugin_textdomain;
use function register_activation_hook;
use function register_uninstall_hook;
use function update_option;
use function wp_enqueue_script;
use function wp_enqueue_style;
use function wp_is_mobile;
use function wp_register_script;
use const JCH_PRO;
use const SHORTINIT;
use const WP_USE_THEMES;
class Loader implements LoggerAwareInterface, ContainerAwareInterface
{
    use LoggerAwareTrait;
    use ContainerAwareTrait;
    /**
     * @var Registry|null
     */
    private $params;
    /**
     * @var Admin
     */
    private $admin;
    /**
     * @var PageCache
     */
    private $pageCache;
    /**
     * @var Installer
     */
    private $installer;
    /**
     * @var Updater
     */
    private $updater;
    /**
     * @var Optimize
     */
    private $optimize;
    public function __construct(Registry $params, \JchOptimize\Plugin\Admin $admin, \JchOptimize\Plugin\Installer $installer, ?\JchOptimize\Plugin\Updater $updater, Optimize $optimize, PageCache $pageCache)
    {
        $this->params = $params;
        $this->admin = $admin;
        $this->installer = $installer;
        $this->updater = $updater;
        $this->optimize = $optimize;
        $this->pageCache = $pageCache;
    }
    public function preboot_init()
    {
        if (\false !== ($settings = get_option('jch_options'))) {
            update_option('jch-optimize_settings', $settings);
            delete_option('jch_options');
        }
    }
    public function init()
    {
        $this->runActivationRoutines();
        if (is_admin()) {
            add_action('admin_menu', [$this->admin, 'addAdminMenu']);
            add_action('admin_init', [$this->admin, 'registerOptions']);
            add_filter('plugin_action_links', [$this->admin, 'loadActionLinks'], 10, 2);
        } else {
            $url_exclude = $this->params->get('menuexcludedurl', []);
            /** @var Input $input */
            $input = $this->container->get(Input::class);
            $jch_backend = $input->get('jchbackend');
            if (\defined('WP_USE_THEMES') && WP_USE_THEMES && $jch_backend != 1 && \version_compare(\PHP_VERSION, '7.3.0', '>=') && !\defined('DOING_AJAX') && !\defined('DOING_CRON') && !\defined('APP_REQUEST') && !\defined('XMLRPC_REQUEST') && (!\defined('SHORTINIT') || \defined('SHORTINIT') && !SHORTINIT) && !Helper::findExcludes($url_exclude, SystemUri::toString())) {
                //Disable NextGen Resource Manager; incompatible with plugin
                //add_filter( 'run_ngg_resource_manager', '__return_false' );
                add_action('init', [$this, 'initializeCache'], 0);
                \ob_start([$this, 'runOptimize']);
            }
        }
        add_action('plugins_loaded', [$this, 'loadPluginTextDomain']);
        register_uninstall_hook(JCH_PLUGIN_FILE, [__CLASS__, 'runUninstallRoutines']);
        if ($this->params->get('order_plugin', '1')) {
            add_action('activated_plugin', [$this, 'orderPlugin']);
            add_action('deactivated_plugin', [$this, 'orderPlugin']);
        }
        if (JCH_PRO) {
            $this->loadProUpdater();
            if ($this->params->get('pro_cache_platform', '0')) {
                add_filter('jch_optimize_get_page_cache_id', [$this, 'getPageCacheHash'], 10, 2);
            }
        }
        //Ajax functions
        add_action('wp_ajax_filetree', [$this->admin, 'doAjaxFileTree']);
        add_action('wp_ajax_multiselect', [$this->admin, 'doAjaxMultiSelect']);
        add_action('wp_ajax_optimizeimages', [$this->admin, 'doAjaxOptimizeImages']);
        add_action('wp_ajax_smartcombine', [$this->admin, 'doAjaxSmartCombine']);
        add_action('wp_ajax_configuresettings', [$this->admin, 'doAjaxConfigureSettings']);
    }
    public function runActivationRoutines()
    {
        //Handles activation routines
        register_activation_hook(JCH_PLUGIN_FILE, [$this->installer, 'activate']);
        if (!\file_exists(\JCH_PLUGIN_DIR . 'dir.php')) {
            $this->installer->activate();
        }
    }
    protected function loadProUpdater()
    {
        $this->updater->load();
    }
    public function initializeCache()
    {
        $this->pageCache->initialize();
        $this->pageCache->setCaching();
    }
    public static function runUninstallRoutines()
    {
        delete_option('jch-optimize_settings');
        /** @var Cache $cache */
        $container = Container::getInstance();
        $cache = $container->get(Cache::class);
        $cache->cleanCache();
        Tasks::cleanHtaccess();
    }
    public function runOptimize($html)
    {
        if (!Helper::validateHtml($html)) {
            return $html;
        }
        //need to check this here, it could be set dynamically
        global $jch_no_optimize;
        $disable_logged_in = $this->params->get('disable_logged_in_users', '1');
        //Need to call Utility::isGuest after init has been called
        if ($jch_no_optimize || $disable_logged_in && !Utility::isGuest()) {
            return $html;
        }
        try {
            $this->optimize->setHtml($html);
            $optimizedHtml = $this->optimize->process();
            //required for compatibility with Hide My WP Ghost https://wordpress.org/support/topic/compatibility-with-hide-my-wp-ghost/
            $optimizedHtml = apply_filters('jch_optimize_save_content', $optimizedHtml);
            $this->pageCache->store($optimizedHtml);
        } catch (Exception $e) {
            $this->logger->log(LogLevel::ERROR, $e->getMessage());
            $optimizedHtml = $html;
        }
        return $optimizedHtml;
    }
    public function loadPluginTextDomain()
    {
        load_plugin_textdomain('jch-optimize', \false, \basename(\dirname(JCH_PLUGIN_FILE)) . '/languages');
    }
    public function orderPlugin() : bool
    {
        $active_plugins = (array) get_option('active_plugins', []);
        $order = ['jch-optimize/jch-optimize.php'];
        //Get the plugins in $order that are currently activated
        $order_short_list = \array_intersect($order, $active_plugins);
        //Remove plugins in $order_short_list from list of activated plugins
        $active_plugins_slist = \array_diff($active_plugins, $order_short_list);
        //Merge $order with $active_plugins_list
        $ordered_active_plugins = \array_merge($order_short_list, $active_plugins_slist);
        update_option('active_plugins', $ordered_active_plugins);
        return \true;
    }
    public function getPageCacheHash($parts)
    {
        if (wp_is_mobile()) {
            $parts[] = '_MOBILE_';
        }
        return $parts;
    }
}
